/*
** lcd.c
**
** LCD 3310 driver
** Unbuffered version - very small memory footprint
** Target: ATMEGA128 :: AVR-GCC
**
** Written by Tony Myatt - 2007
** Quantum Torque - www.quantumtorque.com
*/
#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "lcd.h"

/* delay macro function */
#define lcd_delay() for(i=-32000;i<32000;i++)

/* Command type sent to the lcd */
typedef enum { LCD_CMD  = 0, LCD_DATA = 1 } LcdCmdData;

/* Function prototypes */
void lcd_base_addr(unsigned int addr);
void lcd_send(unsigned char data, LcdCmdData cd);

/* The lcd cursor position */
int lcdCacheIdx;


unsigned char matrix[9][96];	// o matrice care tine ceea ce se afiseaza pe ecran

int piece;						// piesa care se misca
int rotation = 0;				// rotatia acestei piese
// pixelii care compun acea piesa
int x[4];
int y[4];


/* Performs IO & LCD controller initialization */
void lcd_init(void)
{
	int i;
	
	BUTTON_DDR &= (!(B1 | B2 | B3));
	BUTTON_PORT |= (B1 | B2 | B3);
    // Pull-up on reset pin
    LCD_PORT |= LCD_RST_PIN;
	
	// Set output bits on lcd port
	LCD_DDR |= LCD_RST_PIN | LCD_CE_PIN | LCD_DC_PIN | LCD_DATA_PIN | LCD_CLK_PIN;
    
	// Wait after VCC high for reset (max 30ms)
    _delay_ms(15);
    
    // Toggle display reset pin
    LCD_PORT &= ~LCD_RST_PIN;
    lcd_delay();
    LCD_PORT |= LCD_RST_PIN;

    // Disable LCD controller
    LCD_PORT |= LCD_CE_PIN;

    lcd_send(0x21, LCD_CMD);  // LCD Extended Commands
	lcd_send(0x90, LCD_CMD);  // Set LCD Vop(Contrast)
    lcd_send(0x20, LCD_CMD);  // Standard Commands, Horizontal addressing	
    lcd_send(0x0D, LCD_CMD);  // LCD in normal mode	
	
	lcd_clear();
}

/**
 *	Sterge elementele din matrice
 */
void matrixClear()
{
	int i, j;
	
	for(i = 0; i < maxX; i++)
		for(j = 0; j < maxY; j++)
			matrix[j][i] = 0;
}

/**
 *	Functia matrix_init sterge continutul ecranului,
 * afiseaza mesajul de inceput de joc si viteza selectata
 */
void matrix_init(int speed)
{
	int i, j, p;
	matrixClear();
	
	for(i = 0; i < maxX; i++)
		for(j = 0; j < maxY; j++)
			matrix[j][i] = 0;
			
	
	i = 30;
	j = 1;
	
	matrix[j][i++] = 0x01;	//T
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x01;	//T
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F;	//R
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x0D;
	matrix[j][i++] = 0x12;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F;	//I
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x17;	//S
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x1D;
	
	j = 3;
	i = 15;
	
	matrix[j][i++] = 0x1F; //P
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x02;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //R
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x0D;
	matrix[j][i++] = 0x12;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x17;	//S
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x1D;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x17;	//S
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x1D;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;		
	
	matrix[j][i++] = 0x02; //1
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;	
	
	matrix[j][i++] = 0x01;	//T
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x01;	
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //O
	matrix[j][i++] = 0x11;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x00;		

	matrix[j][i++] = 0x17;	//S
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x1D;
	matrix[j][i++] = 0x00;		
	matrix[j][i++] = 0x01;	//T
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x01;	
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1E; //A
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x1E;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //R
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x0D;
	matrix[j][i++] = 0x12;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x01;	//T
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x01;
	matrix[j][i++] = 0x01;	
	matrix[j][i++] = 0x00;	
	
	j = 5;
	i = 15;
	matrix[j][i++] = 0x17;	//S
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x1D;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1F; //P
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x02;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1F; //D
	matrix[j][i++] = 0x11;	
	matrix[j][i++] = 0x0E;	
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x0A;	// :	
	matrix[j][i++] = 0x00;		
	matrix[j][i++] = 0x00;	
	
	
	for(p = 0; p < speed; p++)
	{
		matrix[j][i++] = 0x1F;
		matrix[j][i++] = 0x1F;
		matrix[j][i++] = 0x00;
	}
	
	
}

/**
 *	Functia display afiseaza continutul matricii pe display
 */
void display()
{
	int i, j;
	
	lcd_send(0x80, LCD_CMD);
	lcd_send(0x40, LCD_CMD);
	for(j = 0; j < maxY; j++)
		for(i = 0; i < maxX; i++)
			lcd_send(matrix[j][i], LCD_DATA);
}

/*
void randomPick()
{
	int i = rand() % maxX;
	int j = rand() % maxY;
	
	int val = rand() % 255;
	
	matrix[i][j] = (unsigned char) val;
}
*/


/* Clears the display */
void lcd_clear(void)
{
	int i;
	
	lcd_send(0x80, LCD_CMD);	// x = 0, y = 0   
	lcd_send(0x40, LCD_CMD);	
	
    // Set the entire cache to zero and write 0s to lcd
    for(i = 0; i<96 * 9;i++)
	{
		lcd_send(0x00, LCD_DATA);
    }

}




/* Sends data to display controller */
void lcd_send(unsigned char data, LcdCmdData cd)
{


    if(cd == LCD_DATA)
	{
        LCD_PORT |= LCD_DC_PIN;
    }
	else
	{
        LCD_PORT &= ~(LCD_DC_PIN);
    }	
    // Enable display controller (active low)
    LCD_PORT &= ~LCD_CE_PIN;

    // Either command or data

	
	unsigned char i;
	for(i=0;i<8;i++)
	{
	
		// Set the DATA pin value
		if((data>>(7-i)) & 0x01)
		{
			LCD_PORT |= LCD_DATA_PIN;
		}
		else
		{
			LCD_PORT &= ~LCD_DATA_PIN;
		}
		
		// Toggle the clock
		LCD_PORT |= LCD_CLK_PIN;
		LCD_PORT &= ~LCD_CLK_PIN;
	}

	// Disable display controller
    LCD_PORT |= LCD_CE_PIN;	
}

int nextPixel(int X, int Y)
{
	int i;
	for(i = 0; i < 4; i++)
		if(x[i] == X && y[i] == Y)
			return 1;
	
	return 0;
}


/**
 *	Functia isSetS verifica daca pixelul (x,y)
 * este setat deja.
 */
int isSetS(int x, int y)
{
	if(y % 2 == 0)
	{
		if((matrix[y / 2][x * 4] & 0x0F) == 0)
			return 0;
		return 1;
	}
	else
	{
		if((matrix[y / 2][x * 4] & 0xF0) == 0)
			return 0;
		return 1;
	}
}

/**
 *	Functia isOut verifica daca pixelul (x,y)
 * este in afara matricii.
 */
int isOut(int x, int y)
{
	if(x < 0 || x > 23)
		return 1;
	if(y < 0 || y > 15)
		return 1;
	return 0;
}

/**
 * Functia isSet verifica daca pixelul (x,y) 
 * este ocupat de o alta piesa.
 */
int isSet(int x, int y)
{
	if(nextPixel(x, y) == 1)
		return 0;
		
	if(isOut(x, y))
		return 1;
	
	if(y % 2 == 0)
	{
		if((matrix[y / 2][x * 4] & 0x0F) == 0)
			return 0;
		return 1;
	}
	else
	{
		if((matrix[y / 2][x * 4] & 0xF0) == 0)
			return 0;
		return 1;
	}
}

/**
 *	Functia canMove verifica daca piesa curenta se poate deplasa
 * pe directia data de (dx, dy)
 */
int canMove(int dx, int dy)
{
	int i;
	for(i = 0; i < 4; i++)
	{
		if(isSet(x[i] + dx, y[i] + dy))
			return 0;
		if(isOut(x[i] + dx, y[i] + dy))
			return 0;
	}
	
	return 1;
}

/**
 *	Functia putPixels pune pixelul (x, y) in matrice
 */
void putPixels(int x, int y)
{
	if(y % 2 == 0)
	{
		matrix[y / 2][x * 4]     |= 0x0F;
		matrix[y / 2][x * 4 + 1] |= 0x0F;
		matrix[y / 2][x * 4 + 2] |= 0x0F;
		matrix[y / 2][x * 4 + 3] |= 0x0F;		
	}
	
	else
	{
		matrix[y / 2][x * 4]     |= 0xF0;
		matrix[y / 2][x * 4 + 1] |= 0xF0;
		matrix[y / 2][x * 4 + 2] |= 0xF0;
		matrix[y / 2][x * 4 + 3] |= 0xF0;	
	}
}

/**
 *	Functia erasePixels sterge pixelul (x,y) din matrice
 */
void erasePixels(int x, int y)
{
	if(y % 2 == 0)
	{
		matrix[y / 2][x * 4]     &= 0xF0;
		matrix[y / 2][x * 4 + 1] &= 0xF0;
		matrix[y / 2][x * 4 + 2] &= 0xF0;
		matrix[y / 2][x * 4 + 3] &= 0xF0;		
	}
	
	else
	{
		matrix[y / 2][x * 4]     &= 0x0F;
		matrix[y / 2][x * 4 + 1] &= 0x0F;
		matrix[y / 2][x * 4 + 2] &= 0x0F;
		matrix[y / 2][x * 4 + 3] &= 0x0F;	
	}	
}

/**
 *	Functia putShape pune fiecare pixel al piesei in matrice
 */
void putShape()
{
	int i;
	for(i = 0; i < 4; i++)
		putPixels(x[i], y[i]);
}

/**
 *	Functia eraseShape sterge fiecare pixel al piesei din matrice
 */
void eraseShape()
{
	int i;
	for(i = 0; i < 4; i++)
		erasePixels(x[i], y[i]);
}

/**
 *	Functia move muta piesa pe directia data de (dx, dy) daca
 * aceasta mutare este posibila
 */
int move(int dx, int dy)
{
	int i;
	
	if(!canMove(dx, dy))
		return 0;
		
	eraseShape();
		
	for(i = 0; i < 4; i++)
	{
		x[i] += dx;
		y[i] += dy;
	}
	
	putShape();
	return 1;
}

/**
 *	Functia createRandomPiece creeaza aleator o noua piesa.
 */
int createRandomPiece()
{
	int i = rand() % 7;
	
	piece = i;
	rotation = 0;
		
	if(piece == 0)	//patrat
	{
		if(isSetS(12, 0) || isSetS(12, 1) || isSetS(13, 1) || isSetS(13, 0))
			return 0;	
		x[0] = 12;
		x[1] = 12;
		x[2] = 13;
		x[3] = 13;
		y[0] = 0;
		y[1] = 1;
		y[2] = 1;
		y[3] = 0;
		
	}
	
	if(piece == 1)	//L
	{
		if(isSetS(12, 0) || isSetS(12, 1) || isSetS(12, 2) || isSetS(13, 2))
			return 0;		
		x[0] = 12;
		x[1] = 12;
		x[2] = 12;
		x[3] = 13;
		y[0] = 0;
		y[1] = 1;
		y[2] = 2;
		y[3] = 2;
	}

	if(piece == 2)	//L intors
	{
		if(isSetS(12, 0) || isSetS(12, 1) || isSetS(12, 2) || isSetS(11, 2))
			return 0;		
		x[0] = 12;
		x[1] = 12;
		x[2] = 12;
		x[3] = 11;
		y[0] = 0;
		y[1] = 1;
		y[2] = 2;
		y[3] = 2;	
	}	
	
	if(piece == 3)	//I
	{
		if(isSetS(12, 0) || isSetS(12, 1) || isSetS(12, 2) || isSetS(12, 3))
			return 0;		
		x[0] = 12;
		x[1] = 12;
		x[2] = 12;
		x[3] = 12;
		y[0] = 0;
		y[1] = 1;
		y[2] = 2;
		y[3] = 3;	
	}	
	
	if(piece == 4)	//.I.
	{
		if(isSetS(11, 1) || isSetS(12, 0) || isSetS(12, 1) || isSetS(13, 1))
			return 0;		
		x[0] = 11;
		x[1] = 12;
		x[2] = 12;
		x[3] = 13;
		y[0] = 1;
		y[1] = 0;
		y[2] = 1;
		y[3] = 1;	
	}	
	
	if(piece == 5)	//.I`
	{
		if(isSetS(11, 1) || isSetS(12, 0) || isSetS(12, 1) || isSetS(13, 0))
			return 0;		
		x[0] = 11;
		x[1] = 12;
		x[2] = 12;
		x[3] = 13;
		y[0] = 1;
		y[1] = 0;
		y[2] = 1;
		y[3] = 0;	
	}	
	
	if(piece == 6) //`I.
	{
		if(isSetS(11, 0) || isSetS(12, 0) || isSetS(12, 1) || isSetS(13, 1))
			return 0;		
		x[0] = 11;
		x[1] = 12;
		x[2] = 12;
		x[3] = 13;
		y[0] = 0;
		y[1] = 0;
		y[2] = 1;
		y[3] = 1;
	}
	
	return 1;
}

/**
 *	Functia checkAndErase verifica daca un rand este complet si
 * il sterge.
 */
int checkAndErase()
{
	int i, j, p;
	int work = -10;
	
	for(p = 0; p < 16; p++)
	{
		if(work == (p - 1))
			break;
		work = p;
		for(i = 0; i < 24; i++)		
			if(!isSetS(i, p))
			{
				work = -10;
				break;
			}
	}
	
	if(work == -10)
		return 0 ;
			
	for(i = 0; i < maxX / 4; i++)
	{
		erasePixels(i, work);
	}
	
	for(i = 0; i < maxX / 4; i++)
		for(j = work - 1; j >= 0; j--)
		{
			if(isSetS(i, j))
			{
				erasePixels(i, j);
				putPixels(i, j + 1);
			}
		}
		
	return 1 + checkAndErase();
}

/**
 *	Functia rotate roteste piesa.
 */
void rotate()
{
	if(piece == 0)
		return;
	
	if(piece == 1)
	{
		if(rotation == 0)
		{
			if(isSet(x[1] + 1, y[1]) || isSet(x[1] - 1, y[1]) || isSet(x[1] - 1, y[1] + 1))
				return;
			rotation = 1;
			eraseShape();
			x[0] = x[1] + 1;
			y[0] = y[1];
			x[2] = x[1] - 1;
			y[2] = y[1];
			x[3] = x[1] - 1;
			y[3] = y[1] + 1;
			putShape();
			return;
		}
		
		if(rotation == 1)
		{
			if(isSet(x[1], y[1] - 1) || isSet(x[1], y[1] + 1) || isSet(x[1] - 1, y[1] - 1))
				return;
			rotation = 2;
			eraseShape();
			x[0] = x[1];
			y[0] = y[1] + 1;
			x[2] = x[1];
			y[2] = y[1] - 1;
			x[3] = x[1] - 1;
			y[3] = y[1] - 1;
			putShape();
			return;
		}
		
		if(rotation == 2)
		{
			if(isSet(x[1] + 1, y[1]) || isSet(x[1] - 1, y[1]) || isSet(x[1] + 1, y[1] - 1))
				return;
			rotation = 3;
			eraseShape();
			x[0] = x[1] - 1;
			y[0] = y[1];
			x[2] = x[1] + 1;
			y[2] = y[1];
			x[3] = x[1] + 1;
			y[3] = y[1] - 1;
			putShape();
			return;
		}
		
		if(rotation == 3)
		{
			if(isSet(x[1], y[1] + 1) || isSet(x[1], y[1] - 1) || isSet(x[1] + 1, y[1] + 1))
				return;
			rotation = 0;
			eraseShape();
			x[0] = x[1];
			y[0] = y[1] - 1;
			x[2] = x[1];
			y[2] = y[1] + 1;
			x[3] = x[1] + 1;
			y[3] = y[1] + 1;
			putShape();
			return;
		}		
	}
	
	if(piece == 2)
	{
		if(rotation == 0)
		{
			if(isSet(x[1] + 1, y[1]) || isSet(x[1] - 1, y[1]) || isSet(x[1] - 1, y[1] - 1))
				return;
			rotation = 1;
			eraseShape();
			x[0] = x[1] + 1;
			y[0] = y[1];
			x[2] = x[1] - 1;
			y[2] = y[1];
			x[3] = x[1] - 1;
			y[3] = y[1] - 1;
			putShape();
			return;
		}
		
		if(rotation == 1)
		{
			if(isSet(x[1], y[1] - 1) || isSet(x[1], y[1] + 1) || isSet(x[1] + 1, y[1] - 1))
				return;
			rotation = 2;
			eraseShape();
			x[0] = x[1];
			y[0] = y[1] + 1;
			x[2] = x[1];
			y[2] = y[1] - 1;
			x[3] = x[1] + 1;
			y[3] = y[1] - 1;
			putShape();
			return;
		}
		
		if(rotation == 2)
		{
			if(isSet(x[1] + 1, y[1]) || isSet(x[1] - 1, y[1]) || isSet(x[1] + 1, y[1] + 1))
				return;
			rotation = 3;
			eraseShape();
			x[0] = x[1] - 1;
			y[0] = y[1];
			x[2] = x[1] + 1;
			y[2] = y[1];
			x[3] = x[1] + 1;
			y[3] = y[1] + 1;
			putShape();
			return;
		}
		
		if(rotation == 3)
		{
			if(isSet(x[1], y[1] + 1) || isSet(x[1], y[1] - 1) || isSet(x[1] - 1, y[1] + 1))
				return;
			rotation = 0;
			eraseShape();
			x[0] = x[1];
			y[0] = y[1] - 1;
			x[2] = x[1];
			y[2] = y[1] + 1;
			x[3] = x[1] - 1;
			y[3] = y[1] + 1;
			putShape();
			return;
		}		
	}
	
	if(piece == 3)
	{
		if(rotation == 0)
		{
			if(isSet(x[1] - 1, y[1]) || isSet(x[1] + 1, y[1]) || isSet(x[1] + 2, y[1]))
				return;	
			rotation = 1;
			eraseShape();
			x[0] = x[1] - 1;
			y[0] = y[1];
			x[2] = x[1] + 1;
			y[2] = y[1];
			x[3] = x[1] + 2;
			y[3] = y[1];
			putShape();
			return;	
		}
		
		if(rotation == 1)
		{
			if(isSet(x[1], y[1] - 1) || isSet(x[1], y[1] + 1) || isSet(x[1], y[1] + 2))
				return;	
			rotation = 0;
			eraseShape();
			x[0] = x[1];
			y[0] = y[1] - 1;
			x[2] = x[1];
			y[2] = y[1] + 1;
			x[3] = x[1];
			y[3] = y[1] + 2;
			putShape();
			return;	
		}		
	}
	
	if(piece == 4)
	{
		if(rotation == 0)
		{
			if(isSet(x[2], y[2] + 1))
				return;
			rotation = 1;
			eraseShape();
			x[0] = x[2];
			y[0] = y[2] + 1;
			putShape();
			return;
		}
		
		if(rotation == 1)
		{
			if(isSet(x[2] - 1, y[2]))
				return;
			rotation = 2;
			eraseShape();
			x[1] = x[2] - 1;
			y[1] = y[2];
			putShape();
			return;
		}

		if(rotation == 2)
		{
			if(isSet(x[2], y[2] - 1))
				return;
			rotation = 3;
			eraseShape();
			x[3] = x[2];
			y[3] = y[2] - 1;
			putShape();
			return;
		}

		if(rotation == 3)
		{
			if(isSet(x[2] + 1, y[2]))
				return;
			rotation = 0;
			eraseShape();
			x[0] = x[1];
			y[0] = y[1];
			x[1] = x[3];
			y[1] = y[3];
			x[3] = x[2] + 1;
			y[3] = y[2];
			putShape();
			return;
		}		
	}
	
	if(piece == 5)
	{
		if(rotation == 0)
		{
			if(isSet(x[2] + 1, y[2]) || isSet(x[2] + 1, y[2] + 1))
				return;
			rotation = 1;
			eraseShape();
			x[0] = x[2];
			y[0] = y[2] - 1;
			x[1] = x[2] + 1;
			y[1] = y[2];
			x[3] = x[2] + 1;
			y[3] = y[2] + 1;
			putShape();
			return;
		}
		
		if(rotation == 1)
		{
			if(isSet(x[2] - 1, y[2]) || isSet(x[2] + 1, y[2] - 1))
				return;
			rotation = 0;
			eraseShape();
			x[0] = x[2] - 1;
			y[0] = y[2];
			x[1] = x[2];
			y[1] = y[2] - 1;
			x[3] = x[2] + 1;
			y[3] = y[2] - 1;
			putShape();
			return;
		}		
	}
	
	if(piece == 6)
	{
		if(rotation == 0)
		{
			if(isSet(x[2], y[2] + 1) || isSet(x[2] + 1, y[2] - 1))
				return;
			rotation = 1;
			eraseShape();
			x[0] = x[2] + 1;
			y[0] = y[2] - 1;
			x[1] = x[2] + 1;
			y[1] = y[2];
			x[3] = x[2];
			y[3] = y[2] + 1;
			putShape();
			return;
		}
		
		if(rotation == 1)
		{
			if(isSet(x[2], y[2] - 1) || isSet(x[2] - 1, y[2] - 1))
				return;
			rotation = 0;
			eraseShape();
			x[0] = x[2] - 1;
			y[0] = y[2] - 1;
			x[1] = x[2];
			y[1] = y[2] - 1;
			x[3] = x[2] + 1;
			y[3] = y[2];
			putShape();
			return;
		}		
	}
}

/**
 *	Functia endGame afiseaza mesajul de sfarsit de joc
 * si scorul.
 */
void endGame(int score)
{
	matrixClear();
	lcd_clear();
	
	int j = 3;
	int i = 15;
	
	matrix[j][i++] = 0x1F;	//G
	matrix[j][i++] = 0x11;	
	matrix[j][i++] = 0x1D;		
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1E; //A
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x1E;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1F; //M	
	matrix[j][i++] = 0x02;
	matrix[j][i++] = 0x04;
	matrix[j][i++] = 0x02;	
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x00;		
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;	
	
	matrix[j][i++] = 0x1F; //O
	matrix[j][i++] = 0x11;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x03; //V
	matrix[j][i++] = 0x0C;
	matrix[j][i++] = 0x10;
	matrix[j][i++] = 0x0C;
	matrix[j][i++] = 0x03;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;
	matrix[j][i++] = 0x1F; //R
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x0D;
	matrix[j][i++] = 0x12;
	matrix[j][i++] = 0x00;
	
	j = 5;
	i= 15;	
	
	matrix[j][i++] = 0x17;	//S
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x1D;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1F;	//C
	matrix[j][i++] = 0x11;	
	matrix[j][i++] = 0x11;		
	matrix[j][i++] = 0x00;		
	matrix[j][i++] = 0x1F; //O
	matrix[j][i++] = 0x11;
	matrix[j][i++] = 0x1F;
	matrix[j][i++] = 0x00;		
	matrix[j][i++] = 0x1F; //R
	matrix[j][i++] = 0x05;
	matrix[j][i++] = 0x0D;
	matrix[j][i++] = 0x12;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x1F; //E
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x15;
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x00;	
	matrix[j][i++] = 0x0A;	// :	
	matrix[j][i++] = 0x00;		
	matrix[j][i++] = 0x00;	


	if(score == 0)
	{
		matrix[j][i++] = 0x1F; //O
		matrix[j][i++] = 0x11;
		matrix[j][i++] = 0x1F;
		matrix[j][i++] = 0x00;	
		return;
	}
	
	int cat;
	int rest;
	while(score != 0)
	{
		cat = score / 10;
		rest = score % 10;
		score = cat;
		if(rest == 0)
		{
			matrix[j][i++] = 0x1F; //O
			matrix[j][i++] = 0x11;
			matrix[j][i++] = 0x1F;
			matrix[j][i++] = 0x00;				
			continue;
		}
		
		if(rest == 1)
		{
			matrix[j][i++] = 0x04; //1
			matrix[j][i++] = 0x02;
			matrix[j][i++] = 0x1F;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 2)
		{
			matrix[j][i++] = 0x1D; //2
			matrix[j][i++] = 0x15;
			matrix[j][i++] = 0x17;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 3)
		{
			matrix[j][i++] = 0x15; //3
			matrix[j][i++] = 0x15;
			matrix[j][i++] = 0x1F;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 4)
		{
			matrix[j][i++] = 0x07; //4
			matrix[j][i++] = 0x04;
			matrix[j][i++] = 0x1C;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 5)
		{
			matrix[j][i++] = 0x17; //5
			matrix[j][i++] = 0x15;
			matrix[j][i++] = 0x1D;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 6)
		{
			matrix[j][i++] = 0x1F; //6
			matrix[j][i++] = 0x15;
			matrix[j][i++] = 0x1C;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 7)
		{
			matrix[j][i++] = 0x01; //7
			matrix[j][i++] = 0x01;
			matrix[j][i++] = 0x1F;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 8)
		{
			matrix[j][i++] = 0x1F; //8
			matrix[j][i++] = 0x15;
			matrix[j][i++] = 0x1F;
			matrix[j][i++] = 0x00;				
			continue;
		}

		if(rest == 9)
		{
			matrix[j][i++] = 0x07; //9
			matrix[j][i++] = 0x05;
			matrix[j][i++] = 0x1F;
			matrix[j][i++] = 0x00;				
			continue;
		}

		
	}
}


