/************************************************************************ *********
 * Nicolae Ciobanu
 * Tetris
 *
 *********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "lcd.h"

/**
 *	Functia buttonIsPressed verifica daca un buton este apasat.
 */
int buttonIsPressed()
{
	if((BUTTON_PIN & B1) == 0)
		return 1;
	if((BUTTON_PIN & B2) == 0)
		return 2;
	if((BUTTON_PIN & B3) == 0)
		return 3;
		
	return 0;
}

int main(void)
{	
	int rc = 1;
	int button;
	int i;
	int score = 0;
	int speed = 1;
	int p;
	// Setup LCD
	lcd_init();
	
	matrixClear();
	matrix_init(1);	// mesajul initial
	
	display();
	
	// cresterea vitezei si inceputul jocului
	while(1)
	{
		button = buttonIsPressed();
		if(button == 1)
			break;
		if(button == 2 && speed > 1)
		{
			speed--;
			matrix_init(speed);
			display();
			_delay_ms(10000);
		}
		if(button == 3 && speed < 5)
		{
			speed++;
			matrix_init(speed);
			display();
			_delay_ms(10000);
		}
	}
	_delay_ms(40000);

	matrixClear();
	lcd_clear();
	createRandomPiece();	// se pune prima piesa pe ecran	
	putShape();
	display();
	_delay_ms(10000);
	
	// jocul
	while(1)
	{
		if(move(0, 1) == 0)	// se incearca mutarea in jos
		{
			// mutarea nu a putut fi efectuata
			rc = createRandomPiece();	// se creeaza o noua piesa
			putShape();			
		}
			
		// sfarsitul unui joc
		if(rc == 0)
		{
			// piesa noua nu a putut fi pusa
			endGame(score);	// mesajul de final de joc
			display();
			
			for(p = 0; p < 20; p++)
				_delay_ms(10000);
		
			score = 0;
			speed = 1;
			lcd_clear();			
			matrixClear();
			matrix_init(speed);	// mesajul de inceput de joc
			
			display();
			// inceputul jocului si cresterea vitezei
			while(1)
			{
				button = buttonIsPressed();
				if(button == 1)
					break;
				if(button == 2 && speed > 1)
				{	
					speed--;
					matrix_init(speed);
					display();
					_delay_ms(10000);
				}
				if(button == 3 && speed < 5)
				{
					speed++;
					matrix_init(speed);
					display();
					_delay_ms(10000);
				}
			}
			_delay_ms(40000);
			matrixClear();	
			lcd_clear();		
			
			createRandomPiece();	// se pune o noua piesa
			putShape();
			display();
			_delay_ms(10000);
			rc = 1;
			continue;
		}
		
		display();
		// se testeaza daca butoanele sunt apasate
		for(i = 0; i < 2 * (6 - speed) + 2; i++)
		{
			rc = buttonIsPressed();
			if(rc == 0)	// nu a fost apasat un buton
			{
				_delay_ms(1000);
				continue;
			}
			
			if(rc == 2)	// a fost apasat butonul pentru rotatie
			{
				rotate();
			}
			
			if(rc == 1)	// a fost apasat butonul pentru mutare stanga
				move(-1, 0);
				
			if(rc == 3)	// a fost apasat butonul pentru mutare dreapta
				move(1, 0);
			display();
			_delay_ms(1000);
		}
		
		score += checkAndErase();
		rc = 1;		
	}
	
	return 0;
}